#! /bin/env python
#
#
#  example codes to manipulate the flux catalog
#  $MIRCAT/FluxSource.cat
#
#  todo:
#   - merge in the old ideas from plot_swflux.py
#   - wrap this all in a class
#   - need MJD converter, there's something in datetime now ?

import datetime
import numpy as np
import matplotlib
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.mlab as mlab


class Flux(object):
    def __init__(self,file):
        self.file = file
        f = open(file)
        l = f.readlines()
        f.close()
        self.sources=[]
        self.source=[]
        self.time=[]
        freq = []
        flux = []
        err  = []
        for line in l:
            # lines starting with '##' store the aliases, we should remember them
            if line[0] == '#': continue
            w = line.split()
            if len(w) < 5: continue
            if not self.sources.count(w[0]):
                self.sources.append(w[0])
            ymd = datetime.datetime.strptime(w[1],"%Y-%b-%d.%H")
            self.source.append(w[0])
            self.time.append(ymd)
            freq.append(float(w[2]))
            flux.append(float(w[3]))
            err.append(float(w[4]))
        self.freq = np.array(freq)
        self.flux = np.array(flux)
        self.err = np.array(err)
        print "Found %d entries in %s" % (len(self.time),file)
        print "Found %d sources in %s" % (len(self.sources),file)

    def getSome(self,source):
        n=len(self.time)
        print "Source: ",source
        print "n: ",n
        for i in range(0,10):
            # print self.freq[i]
            print i
        print "Done."
        

            
    def filename(self):
        return self.file

        


def flux(file):
    """ read carma fluxes
    samples (only first 5 columns count)
    1510M089     1988-NOV-22.0    90.0     1.40     0.20
    1512-090     2008-NOV-01.0    96.3     1.74     0.26   CARMA8       3C273        16.6
    """
    f = open(file)
    l = f.readlines()
    f.close()
    time = []
    freq = []
    flux = []
    err  = []
    for line in l:
        if line[0] == '#': continue
        w = line.split()
        if len(w) < 5:
            print "Bad line:",line
            continue
        ymd = datetime.datetime.strptime(w[1],"%Y-%b-%d.%H")
        time.append(ymd)
        freq.append(float(w[2]))
        flux.append(float(w[3]))
        err.append(float(w[4]))
    return (time,np.array(flux),np.array(err))


def try1(file='PKS_1510-089.flux'):
    (t,f,df) = flux(file)
    flo = f - df
    fhi = f + df

    fig = plt.figure()
    ax = fig.add_subplot(111)


    years    = mdates.YearLocator()   # every year
    months   = mdates.MonthLocator()  # every month
    yearsFmt = mdates.DateFormatter('%Y')




    ax.plot_date(t, f, 'ro')
    ax.vlines(t,f,fhi)
    #ax.hlines(flo,t-0.25,t+0.25)

    ax.vlines(t,f,flo)
    #ax.hlines(fhi,t-0.25,t+0.25)

    # format the ticks
    ax.xaxis.set_major_locator(years)
    ax.xaxis.set_major_formatter(yearsFmt)
    ax.xaxis.set_minor_locator(months)
    ax.autoscale_view()

    # format the coords message box
    def fscale(x): return '%1.2f Jy'%x
    ax.fmt_xdata = mdates.DateFormatter('%Y-%m-%d')
    ax.fmt_ydata = fscale
    ax.grid(True)

    # set some labels
    ax.set_xlabel("Time")
    ax.set_ylabel("Flux (Jy)")
    ax.set_title(file)

    fig.autofmt_xdate()
    plt.show()
    plt.savefig('%s.ps' % file)

if __name__ == '__main__':
    try1('3c84.flux')
