import scipy.interpolate as interp
import scipy.stats as stat
import matplotlib.pyplot as plt
import carmatable as ct
import scipy
import linreg
import pacs
import os
import numpy as np
import parfile
import pacsplot
import time
import calendar
def inRanges(list,ranges):
    """ takes a list of floats and a list of tuples of floats,
    and returns all the floats that fall within one of the tuples.
    """
    output=[]
    for item in list:
        valid=False
        for min,max in ranges:
            if min<=item<=max:
                output.append(item)
                break
    return output
def getSourceIntervals(project):
    """ Gets the intervals over which a partiular source is being observed.
    """
    toHours=lambda x:float(x[0:2])+float(x[2:4])/60+float(x[4:])/3600
    sources=ct.CarmaTable()
    sources.open("%s.txt"%project)
    sourceList=set(zip(sources.getCol("TargetC"),sources.getCol("TargetS")))
    output=[]
    for car,sza in sourceList:
        output.append([car,sza])
        for i in range(sources.getNrows()):    
            row=sources.getRow(i)
            if row[2]==car:
                output[-1].append((toHours(row[0]),toHours(row[1])))
    return output
def merge(plotType):
    os.system("pdftk `ls | grep %s0` cat output %smerged.pdf"%(plotType,plotType))
    os.system("rm %s0*"%plotType)
def decitime(times):
    """Converts a list of hh:mm:ss times to a list of decimal hours.
    """
    output=[]
    for time in times:
        temp=time.split(':')
        output.append(float(temp[0])+float(temp[1])/60.0+float(temp[2])/3600.0)
    return output
def bestWrap(carList,szaList,ratio=3):
    """This program tries to choose the optimal wrap for carma data. For
    example, if the SZA phase was 30, and the expected ratio was 3, we
    would expect a phase of 90. However, this may be recorded as -270,
    and therefore should be converted to 90.
    """
    outputS=[]
    outputC=[]
    for (car,sza) in zip(carList,szaList):
        diff=abs(car-sza*ratio)
        if abs((car+360.0)-sza*ratio)<diff:
            outputS.append(sza)
            outputC.append(car+360)
        elif abs((car-360.0)-sza*ratio)<diff:
            outputS.append(sza)
            outputC.append(car-360)
        else:
            outputS.append(sza)
            outputC.append(car)
    return (outputC,outputS)
def makeTables(array,project,cFreq,sFreq,date,badAnts=[]):
    """
    Generates 1 plot for every baseline and plots the SZA and CARMA
    phases created by the pipeline.
    """
    sTable=ct.CarmaTable()
    cTable=ct.CarmaTable()
#    sTable.open("szaphaseDiffs.txt")
    sTable.open("szaphaseDiffsInterpolated.txt")
    cTable.open("carmaphaseDiffs.txt")
    sBaselines=sTable.getColNames()[1:]
    cBaselines=cTable.getColNames()[1:]
    sTime=np.array(decitime(sTable.getCol(0)))
    cTime=np.array(decitime(cTable.getCol(0)))
    sData=np.array(sTable.splat())[:,1:]
    cData=np.array(cTable.splat())[:,1:]
#    sData=np.array(sData,int)
#    cData=np.array(cData,int)
    sData=np.array(sData,float)
    cData=np.array(cData,float)
    min=-1
    max=len(cTime)
    for i in range(0,len(cTime)):
        if cTime[i]<sTime[0]:
            min=i
        if cTime[i]>sTime[-1]:
            max=i
    cTime=cTime[:max]
    cData=cData[:max]
    cTime=cTime[min+1:]
    cData=cData[min+1:]
    sPaths=[]
    cPaths=[]
    filteredSBaselines=[]
    filteredCBaselines=[]
    print 'badAnts:',badAnts
    for sCol,cCol,sBaseline,cBaseline in zip(sData.transpose(),cData.transpose(),sBaselines,cBaselines):
        good=True
        for ant in badAnts:
            print "is %s in %s or %s?"%(ant,sBaseline,cBaseline)
            if str(ant) in cBaseline.split('-')+sBaseline.split('-'):
                good=False
                break
        if good:
            filteredSBaselines.append(sBaseline)
            filteredCBaselines.append(cBaseline)
            #for line in cTime:
            #    print line
            sPaths.append(interp.interp1d(sTime,sCol))#This function traces the path of SZA
            cPaths.append(interp.interp1d(cTime,cCol))#This function traces the path of CARMA
        #Although no interpolation is needed, source filtering is now by time only
    os.system("mkdir -p phasediffs")
    for element in getSourceIntervals(project):
        #print element[0]
        #print element[2:]
        #for t in sTime:
        #    print "%-20s\t"%t,
        #print ""
        #print inRanges(sTime,element[2:])
        getPairs(element[0],element[1],inRanges(sTime,element[2:]),cPaths,sPaths,filteredSBaselines,filteredCBaselines,cFreq,sFreq,date)
def getBrightness(source,cFreq,day,freqPriority=1):
    """freqPriority is the number of GHz it is worth to be 1 day closer"""
    #Date is of format 99DEC31
    # TODO deal with the asymm bug in calflux?
    cmd = "calflux source=%s freq=%f delfreq=50"%(source,cFreq)
    cmd = "calflux source=%s freq=%f date=%s deldate=200 delfreq=50"%(source,cFreq,day)
    print "CALFLUX: ",cmd
    day=calendar.timegm(time.strptime(day.split('.')[0],"%y%b%d"))
    fin=os.popen(cmd)
    text=fin.readlines()[1:]
    text=[line.split() for line in text]
    records=[(calendar.timegm(time.strptime(line[3].split('.')[0],"%y%b%d")),float(line[5]),float(line[7])) for line in text]
    best=(0,float('inf'))
    for record in records:
        score=abs((record[0]-day))*freqPriority/(24*3600)+abs(cFreq-record[1])
        if score<best[1]:
            best=(record[2],score)
    print "BEST FLUX found for %s: %f Jy" % (source,best[0])
    return best[0]
def getPairs(cSource,sSource,time,cPaths,sPaths,sBaselines,cBaselines,cFreq,sFreq,date):
    """
    This function will generate a table containing:
    return allSData,allCData
    The source name
    The baselines
    A list of phases
    """
    "calflux source=%s freq=%.2f deldate=100 delfreq=50"%(cSource,cFreq)
    shortFloat=lambda x:["%.1f"%y for y in x]
    nBaselines=(2*len(cBaselines))
    names=["Time"]+[""]*nBaselines
    names[1::2]=cBaselines
    names[2::2]=sBaselines
    table=ct.CarmaTable(names,["s"]+["i"]*nBaselines,[""]*(nBaselines+1))
    table.editCol(time,0)
    allCData=[]
    allSData=[]
    for cPath,sPath,cBaseline,sBaseline in zip(cPaths,sPaths,cBaselines,sBaselines):
        # print "Retrieving %s"%time
        cData=cPath(time)
        sData=sPath(time)
        #(cData,sData)=bestWrap(cData,sData,cFreq/sFreq)
        allCData.extend(cData)
        allSData.extend(sData)
        table.editCol(shortFloat(cPath(time)),cBaseline)
        table.editCol(shortFloat(sPath(time)),sBaseline)
    slope=linreg.linreg(allSData,allCData)
    rmsCar=pacsplot.stdev(allCData)
    rmsDiff=pacsplot.stdev([(cFreq/sFreq)*s-c for s,c in zip(allSData,allCData)])
    table.setDict({"CARSource":cSource,\
        "SZASource":sSource,\
        "slope":slope[0],\
        #The probability that predicted is the real one.
        "RMSImprovement":(rmsCar-rmsDiff),\
        "Brightness":getBrightness(cSource,cFreq,date)})
    table.write("phasediffs/%s.txt"%cSource)
def plots(array,project,badAnts=[]):
    """
    Generates 1 plot for every baseline and plots the SZA and CARMA
    phases created by the pipeline.
    """
    p=parfile.ParFile("drpacs.def")
    cFreq=float(p.get("cFreq"))
    sFreq=float(p.get("sFreq"))
    date=p.get("date")
    p.save()
    makeTables(array,project,cFreq,sFreq,date,badAnts)
    allfiles=[]
    for root, dirs, files in os.walk("phasediffs"):
        allfiles=allfiles+files
    tables=dict([(y[:-4],ct.CarmaTable()) for y in allfiles])
    colors="kgycmbr"*(1+len(tables)/7)
    for key in tables.keys():
        tables[key].open("phasediffs/"+key+".txt")
    names=tables[tables.keys()[0]].getColNames()
    #print zip(names[2::2],names[1::2])
    count=0
    debug=[]
    for sBaseline,cBaseline in zip(names[2::2],names[1::2]):
        count+=1
        figAll=plt.figure(figsize=(14,11), dpi=90)
        axAll=figAll.add_subplot(1,1,1)
        allSPhases=[]
        allCPhases=[]
        for key,color in zip(tables.keys(),colors):
            #This will print a plot for each source and baseline
            fig2=plt.figure(figsize=(14,11), dpi=90)
            ax2=fig2.add_subplot(1,1,1)
            thisS=tables[key].getFloatCol(sBaseline)
            thisC=tables[key].getFloatCol(cBaseline)
            try:
                fitLine,envelope,fit=pacsplot.linregLists(thisS,thisC)
            except IndexError:
                print "### Warning [PVP plot] Baseline %s or %s failed for %s"%(sBaseline,cBaseline,key)
                plt.close(fig2)
                continue
            xMin=np.min(thisS)
            xMax=np.max(thisS)
            linex=np.array([xMin,xMax])
            predicted=linex*(cFreq/sFreq)
            ax2.plot(fitLine[0],fitLine[1],'-b',label="Fit:%.1fs%+.1f"%tuple(fit[:2]))
            ax2.fill(envelope[0],envelope[1],'b',label="Fit error",alpha=.3)
            ax2.plot(linex,predicted,'--r',label="Expected fit %.1f"%(cFreq/sFreq))
            ax2.plot(thisS,thisC,'.%s'%color,label=key)
            ax2.legend(loc='lower right')
            ax2.set_xlabel("SZA Phase on baseline %s"%sBaseline)
            ax2.set_ylabel("CARMA Phase on baseline %s"%cBaseline)
            ax2.set_title("Phase vs Phase for %s"%(project))
            fig2.savefig("phaseplot%s%03i.pdf"%(key,count), orientation='landscape', bbox_inches='tight', pad_inches=1)
            #Now working on the all sources plot
            allSPhases.extend(thisS)
            allCPhases.extend(thisC)
            axAll.plot(thisS,thisC,'.%s'%color,label=key)
            debug.append(thisC)
            plt.close(fig2)
        try:
            fitLine,envelope,fit=pacsplot.linregLists(allSPhases,allCPhases)
        except IndexError:
            print "### Warning [PVP plot] Baseline %s or %s has failed for all sources"%(sBaseline,cBaseline)
            plt.close(figAll)
            continue
        xMin=np.min(allSPhases)
        xMax=np.max(allSPhases)
        linex=np.array([xMin,xMax])
        predicted=linex*(cFreq/sFreq)
        axAll.plot(fitLine[0],fitLine[1],'-b',label="Fit:%.1fs%+.1f"%tuple(fit[:2]))
        axAll.fill(envelope[0],envelope[1],'b',label="Fit error",alpha=.3)
        axAll.plot(linex,predicted,'--r',label="Expected fit %.1f"%(cFreq/sFreq))
        axAll.legend(loc='lower right')
        axAll.set_xlabel("SZA Phase on baseline %s"%sBaseline)
        axAll.set_ylabel("CARMA Phase on baseline %s"%cBaseline)
        axAll.set_title("Phase vs Phase for %s"%(project))
        figAll.savefig("phaseplot%03i.pdf"%(count), orientation='landscape', bbox_inches='tight', pad_inches=1)
        plt.close(figAll)
    for key in tables.keys():
        merge("phaseplot%s"%key)
    merge("phaseplot")
    print ""
    return debug
def plotSources(root):
    """Plots every source's ratio between the expected and actual
    phase ratios, as well as the correlation between the SZA and
    CARMA data.
    """
    allfiles=[]
    for root, dirs, files in os.walk(root):
        output=[]
        for file in files:
            if "phasediffs" in root:
                output.append((root,file[:-4]))
        allfiles=allfiles+output
    sources={}
    lastFloat=lambda x:float(x.split()[-1])
    for file in allfiles:
        if not sources.has_key(file[1]):
            sources[file[1]]=[]
        p=parfile.ParFile("%s/../drpacs.def"%file[0])
        cFreq=float(p.get("cFreq"))
        ismm1=(cFreq>120)
        sFreq=float(p.get("sFreq"))
        del p
        table=ct.CarmaTable()
        table.open("%s/%s.txt"%tuple(file))
        #These entries are of the format (slope ratio,P-value,Brightness)
        dict=table.getDict()
        sources[file[1]].append((float(dict["slope"])*sFreq/cFreq,float(dict["RMSImprovement"]),float(dict["Brightness"]),ismm1))
        #print (float(dict["slope"])*sFreq/cFreq,float(dict["RMSImprovement"]),float(dict["Brightness"]),ismm1)
    figAll=plt.figure(figsize=(14,11), dpi=90)
    axAll=figAll.add_subplot(1,1,1,xscale='log')
    xAll=[]
    yAll=[]
    cAll=[]
    figmm1=plt.figure(figsize=(14,11), dpi=90)
    axmm1=figmm1.add_subplot(1,1,1,xscale='log')
    xmm1=[]
    ymm1=[]
    cmm1=[]
    figmm3=plt.figure(figsize=(14,11), dpi=90)
    axmm3=figmm3.add_subplot(1,1,1,xscale='log')
    xmm3=[]
    ymm3=[]
    cmm3=[]
    complimit=1
    for source in sources.keys():
        print "Plotting %s"%source
        fig2=plt.figure(figsize=(14,11), dpi=90)
        ax2=fig2.add_subplot(1,1,1,xscale='log')
        try:
            x,y,color,mm1=zip(*[(float(a),float(b),float(c),d) for (a,b,c,d) in sources[source] if a>0])
        except Exception:
            print "ERROR!"
            continue
        xAll+=x
        yAll+=y
        cAll+=color
        for xI,yI,colorI,mm1I in zip(x,y,color,mm1):
            if mm1I:
                xmm1.append(xI)
                ymm1.append(yI)
                cmm1.append(colorI)
            else:
                xmm3.append(xI)
                ymm3.append(yI)
                cmm3.append(colorI)
        ax2.scatter(x,y,marker='o',label=source)
        limit=max(list(x)+[1/min(list(x))])#This ensures that the plot is symetrical
        complimit=max(limit,complimit)
        ax2.set_xlim(.5/limit,2*limit)
        ax2.set_xlabel("Slope Ratio")
        ax2.set_ylabel("RMSImprovement")
        ax2.set_title(source)
        fig2.savefig("%s.pdf"%source, orientation='landscape', bbox_inches='tight', pad_inches=1)
        plt.close(fig2)
    print "Plotting all sources"
    ####This filters out all Y-values below -180
    #xAll,yAll,cAll=zip(*[(x,y,c) for (x,y,c) in zip(xAll,yAll,cAll) if y>-180])
    #xmm1,ymm1,cmm1=zip(*[(x,y,c) for (x,y,c) in zip(xmm1,ymm1,cmm1) if y>-180])
    #xmm3,ymm3,cmm3=zip(*[(x,y,c) for (x,y,c) in zip(xmm3,ymm3,cmm3) if y>-180])
    ############################################
    figAll.colorbar(axAll.scatter(xAll,yAll,c=cAll,marker='o',cmap=plt.cm.hot),ax=axAll)
    axAll.set_xlim(.5/complimit,2*complimit)
    axAll.set_ylim(-180,max(yAll))
    axAll.set_xlabel("Slope Ratio")
    axAll.set_ylabel("RMSImprovement")
    axAll.set_title("Composite")
    figAll.savefig("Composite.pdf", orientation='landscape', bbox_inches='tight', pad_inches=1)
    plt.close(figAll)
    #############Improvement vs brightness
    figbri=plt.figure(figsize=(14,11), dpi=90)
    axbri=figbri.add_subplot(1,1,1)
    #axbri=figbri.add_subplot(1,1,1,xscale='log')
    axbri.scatter(cmm1,ymm1,marker='o',c='red',label="1 mm")
    axbri.scatter(cmm3,ymm3,marker='o',c='yellow',label="3 mm")
    fitLine1,envelope1,fit=pacsplot.linregLists(cmm1,ymm1,1)
    axbri.plot(fitLine1[0],fitLine1[1],'-r',label="Fit:%.1fJy%+.1f"%tuple(fit[:2]))
    #axbri.fill(envelope1[0],envelope1[1],'r',label="Fit error",alpha=.3)
    fitLine3,envelope3,fit=pacsplot.linregLists(cmm3,ymm3,1)
    axbri.plot(fitLine3[0],fitLine3[1],'-y',label="Fit:%.1fJy%+.1f"%tuple(fit[:2]))
    #axbri.fill(envelope3[0],envelope3[1],'y',label="Fit error",alpha=.3)
    fitLine,envelope,fit=pacsplot.linregLists(cAll,yAll,1)
    axbri.plot(fitLine[0],fitLine[1],'-b',label="Fit:%.1fJy%+.1f"%tuple(fit[:2]))
    #axbri.fill(envelope[0],envelope[1],'b',label="Fit error",alpha=.3)
    axbri.set_ylim(-180,max(yAll))
    axbri.set_xlabel("Brightness")
    axbri.set_ylabel("RMSImprovement")
    axbri.set_title("Composite")
    axbri.legend(loc='lower right')
    figbri.savefig("Brightness.pdf", orientation='landscape', bbox_inches='tight', pad_inches=1)
    plt.close(figbri)
    if xmm1:
        print "Plotting 1 mm sources"
        figmm1.colorbar( axmm1.scatter(xmm1,ymm1,c=cmm1,marker='o',cmap=plt.cm.hot) ,ax=axmm1)
        axmm1.set_xlim(.5/complimit,2*complimit)
        axmm1.set_ylim(-180,max(yAll))
        axmm1.set_xlabel("Slope Ratio")
        axmm1.set_ylabel("RMSImprovement")
        axmm1.set_title("1mm observations")
        figmm1.savefig("mm1.pdf", orientation='landscape', bbox_inches='tight', pad_inches=1)
    plt.close(figmm1)
    if xmm3:
        print "Plotting 3 mm sources"
        figmm3.colorbar( axmm3.scatter(xmm3,ymm3,c=cmm3,marker='o',cmap=plt.cm.hot),ax=axmm3)
        axmm3.set_xlim(.5/complimit,2*complimit)
        axmm3.set_ylim(-180,max(yAll))
        axmm3.set_xlabel("Slope Ratio")
        axmm3.set_ylabel("RMSImprovement")
        axmm3.set_title("mm3 observations")
        figmm3.savefig("mm3.pdf", orientation='landscape', bbox_inches='tight', pad_inches=1)
    plt.close(figmm3)

def plotSourcesOld(root):
    allfiles=[]
    for root, dirs, files in os.walk(root):
        output=[]
        for file in files:
            if "phasediffs" in root:
                output.append((root,file))
        allfiles=allfiles+output
    sources=[x[:-4] for x in list(set(zip(*allfiles)[1]))]
    for source in sources:
        plt.figure(num=1, figsize=(14,11), dpi=90)
        print source
        cPhases=[]
        sPhases=[]
        tableDirs=["%s/%s"%x for x in allfiles if x[1][:-4]==source]
        tables=[]
        for tableDir in tableDirs:
            tables.append(ct.CarmaTable())
            tables[-1].open(tableDir)
        for table in tables:
            cCols=table.getColNames()[1::2]
            print cCols
            sCols=table.getColNames()[2::2]
            for col in cCols:
                cPhases.extend(table.getFloatCol(col))
            for col in sCols:
                sPhases.extend(table.getFloatCol(col))
        if not cPhases:
            continue
        print len(sPhases)
        #for x in zip(sPhases,cPhases):
            #print "%-10f%f"%x
        plt.figure(num=1, figsize=(14,11), dpi=90)
        plt.plot(sPhases,cPhases,'.',label="Phase")
        slope,int,slopeE,intE = [float(x) for x in linreg.linreg(sPhases,cPhases,4)]
        linex=np.array([np.min(sPhases),np.max(sPhases)])
        liney=linex*slope+int
        predicted=linex*3
        plt.plot(linex,liney,'-',label="Fit:%.1fs%+.1f"%tuple(fit))
        plt.plot(linex,predicted,'-',label="Expected fit")
        plt.legend()
        plt.xlabel("SZA Phase")
        plt.ylabel("CARMA Phase")
        plt.title("Phase vs Phase for %s"%source)
        plt.savefig("phaseplot%s.pdf"%source, orientation='landscape', bbox_inches='tight', pad_inches=1)
        plt.close(1)
