#! /bin/csh -f
#
#  Create a coverage of integration times per pointing
#  Needs NEMO, but perhaps varmap(s) can also make this map
#  Needs a special version of uvindex, called uvindex2
#
#  See also the movie maker script, which uses this script
# 
#  march-2011    Created for N1333 project              Peter Teuben
#  9-mar-2011    added to MIS
#
# TODO:
#   - optionally use a minmax for cgdisp
#   - check if varmap(s) can do the same without using NEMO
#   - option for uvindex to avoid the patched uvindex2 version
#     or add select= to uvindex?

if ($#argv == 0) then 
  echo "Usage: $0 xyt1.tab xyt2.tab ..."
  exit 0
endif


if ($?MIR == 0) then
    echo No MIRIAD, trying to load it via astromake
    source /astromake/astromake_start
    astroload miriad
endif

if ($?NEMO == 0) then
    echo No NEMO, trying to load it via astromake
    source /astromake/astromake_start
    astroload nemo
endif


#   some defaults that could be changed
set datamin=100
set datamax=160
set colormap=3


# to create an (dra,ddex,inttime) need to use uvindex2, special clone of uvindex

# e.g.
# uvindex2 vis=23feb.mir | grep ^SVS | awk '{print $4,$5,$6}' > 23feb.pos


# convert all the pos files to a NEMO snapshot
# note the dra,ddec are in arcsec, and we like the numbers in arcmin

rm -rf n1333pos.snap n1333pos.fits n1333pos.mir

cat $* | tabmath - - %1/60,%2/60,%3 all > junk.tab
tabtos junk.tab n1333pos.snap "" x,y,m 
#  the -8:8, 64x64 grid and %1/16 are all related to get them to come out this way
snapgrid n1333pos.snap - all -8:8 -8:8 nx=64 ny=64 |\
  ccdmath - - '%1/16' |\
  ccdfits - n1333pos.fits
fits in=n1333pos.fits out=n1333pos.mir op=xyin
#   mask all 0's
# puthd in=n1333pos.mir/datamin value=0 type=real
#minmax in=n1333pos.mir
immask in=n1333pos.mir options=datamin flag=false
#minmax in=n1333pos.mir
#   plot
cgdisp in=n1333pos.mir device=inttime.gif/gif range=$datamin,$datamax,lin,$colormap labtyp=absnat options=full,wedge
histo in=n1333pos.mir
