#! /bin/csh -f
#
#
#     NGC 1333 automated data reduction pipeline
#
#     this can run as crontab as well
#
# Some sample CPU timings as the project and scripts grew....
#  17-mar: E22    1033.856u 188.876s   28:13.16 72.2%     0+0k 0+0io 86pf+0w
#  20-mar: D3     1405.294u 256.120s   39:45.35 69.6%     0+0k 0+0io 105pf+0w
#  22-mar: D7     1497.138u 271.970s   43:24.87 67.9%     0+0k 0+0io 115pf+0w
#  24-mar: D8     6744.333u 269.045s 2:03:08.79 94.9%     0+0k 0+0io 116pf+0w
#                 (this is now with just continuum mapping each field)
#  25-mar: D9     8044.753u 475.150s 2:33:12.76 92.6%     0+0k 0+0io 311pf+0w   
#  26-mar: D10    7601.259u 493.678s 2:27:51.60 91.2%     0+0k 0+0io 311pf+0w
#  28-mar: D12   13162.906u 610.134s 4:11:58.24 91.1%     0+0k 0+0io 346pf+0w
#  29-mar: D13    7597.150u 606.732s 2:32:22.29 89.7%     0+0k 0+0io 383pf+0w


#   commandline parameters
set today = `date  +%a`
#   this will become $MIS/cat/n1333.lis
unset dirs
#   if you set a pipefile, it will use this instead of the one liners
#   must be an absolute reference, or must live in $MIS/cat
#   args.... @todo in pipe we can't yet pass on parameters that are not pipepars
unset pipefile


#  get MIRIAD if needed
if ($?MIR == 0) then
  source /astromake/astromake_start
  astroload miriad
endif

#  get NEMO if needed
if ($?NEMO == 0) then
  source /astromake/astromake_start
  astroload nemo
endif

# do the work in a fresh directory (using today=test you could override this)

rm -rf Nightly_$today
mkdir  Nightly_$today
rm -f  Nightly_latest
ln -s  Nightly_$today Nightly_latest
cd     Nightly_$today

# get a fresh MIS?

if ($?MIS == 0) then
  cvs -Q co mis
  pushd mis
  ./configure
  source mis_start.csh
  popd
endif

# use the following projects (using dirs= you could override this, use absolute reference)
set dirs=$MIS/cat/n1333.lis 


#    now, finally before we start, parse command line arguments
foreach arg ($*)
  set $arg
end

#  copy the current dirs
if (! -e n1333.lis) cp $MIS/cat/n1333.lis .

#  must ensure a default Nightly link exists for scripts such as do_mos1
if (! -e $MIS/Nightly) then
  echo Creating symlink $MIS/Nightly
  ln -s `pwd` $MIS/Nightly
endif

#  create the empty project directories with the previously pipesave'd MIS files
piperun -c -o pipesetup.log $dirs 'pipepar -c ; pipesetup project=%s'

#  there is no MIS command for this yet, but if you had some new global 
#  parameters in some global mis.def file, this would be the place to
#  apply those, e.g.
#      piperun $dirs "pipepar imsize=128 cell=1 itermax=1000"
#  we might have a new method to shortcut this. 

# get data, make report
piperun -o getdata.log  $dirs 'getdata link=1'
piperun -o report.log   $dirs report


# single dish
piperun -o uvcatSD.log  $dirs do_uvcatSD
piperun -o reduceSD.log $dirs 'do_reduceSD device=/null sleep=0'
piperun -o mapSD.log    $dirs 'do_mapSD device=/null'
piperun -o inttime.log  $dirs  map_inttime
# another median to create final SD cubes
#do_mapSDmedian   >& mapSDmedian.log
do_mapSDmedian4>& mapSDmedian4.log

# ALL
#mk_map_inttime.csh cx323.*/xyt.tab  >& mk_map_inttime.log
# D only
#mk_map_inttime.csh cx323.1D*/xyt.tab >& mk_map_inttime.log

# interferometry
piperun -o uvcat1.log   $dirs do_uvcat1
piperun -o inspect1.log $dirs do_inspect1
piperun -o cal1.log     $dirs 'do_cal1 calflux=12.5'
# piperun -o map1.log     $dirs do_map1

# some QA stuff
mk_gaintabs.csh >& mk_gaintabs.log

# combined maps for continuum and 4 channel versions of the 3 molecules
# in 4 different array and antenna selection choices
do_mos1            >& mos1.log
do_mos1 config=D   >& mos1_D.log
do_mos1 config=E   >& mos1_E.log
do_mos1 ants=LOW   >& mos1_LOW.log


# normally at the very end of all the work, you want to save it
# but in a nightly we don't expect to have to do this, since things
# never really (should) change
# these files are now copied to your $MIS/def tree
piperun -o pipesave.log     $dirs pipesave

# if you are inspecting data, you now probably want to commit the 
# mis.def (and other) files back from $MIS/def into the CVS repository 
# with the 'cvs commit' command, or the shortcut in mis:
echo mis -i
